// cicslog.cpp : implementation file
//

#include "stdafx.h"
#include "loginx.h"
#include "cicslog.h"
// COMMAREA definition for CICS VERIFYPW program
#include "verifypw.h"

// Default name of the CICS Client ini file
#define INIFILENAME "\\cicscli\\bin\\cicscli.ini"
#define MAX_LINE_LEN 512

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Include for CICS ECI functions
#define CICS_W32
#include <cicseci.hpp>

// Include file for subclassed ECI object
#include "clnterr.h"

/////////////////////////////////////////////////////////////////////////////
// cicslog

// The one and only ECI object
fixCclECI cicsECI;

IMPLEMENT_DYNCREATE(cicslog, CCmdTarget)

cicslog::cicslog()
{
	EnableAutomation();
	
	// To keep the application running as long as an OLE automation 
	//	object is active, the constructor calls AfxOleLockApp.
	
	AfxOleLockApp();

	m_numberservers = cicsECI.serverCount();
}

cicslog::~cicslog()
{
	// To terminate the application when all objects created with
	// 	with OLE automation, the destructor calls AfxOleUnlockApp.
	
	AfxOleUnlockApp();
}


void cicslog::OnFinalRelease()
{
	// When the last reference for an automation object is released
	// OnFinalRelease is called.  The base class will automatically
	// deletes the object.  Add additional cleanup required for your
	// object before calling the base class.

	CCmdTarget::OnFinalRelease();
}


BEGIN_MESSAGE_MAP(cicslog, CCmdTarget)
	//{{AFX_MSG_MAP(cicslog)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(cicslog, CCmdTarget)
	//{{AFX_DISPATCH_MAP(cicslog)
	DISP_PROPERTY(cicslog, "clientname", m_clientname, VT_BSTR)
	DISP_PROPERTY(cicslog, "numberservers", m_numberservers, VT_I2)
	DISP_PROPERTY(cicslog, "errmsg", m_errmsg, VT_BSTR)
	DISP_FUNCTION(cicslog, "performVerifyPW", performVerifyPW, VT_I2, VTS_BSTR VTS_BSTR VTS_BSTR)
	DISP_FUNCTION(cicslog, "getServerName", getServerName, VT_BSTR, VTS_I2)
	DISP_FUNCTION(cicslog, "getServerDesc", getServerDesc, VT_BSTR, VTS_I2)
	DISP_FUNCTION(cicslog, "GetClientName", GetClientName, VT_BSTR, VTS_NONE)
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_Icicslog to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {BB765F93-BE9E-11D1-BBC8-204C4F4F5020}
static const IID IID_Icicslog =
{ 0xbb765f93, 0xbe9e, 0x11d1, { 0xbb, 0xc8, 0x20, 0x4c, 0x4f, 0x4f, 0x50, 0x20 } };

BEGIN_INTERFACE_MAP(cicslog, CCmdTarget)
	INTERFACE_PART(cicslog, IID_Icicslog, Dispatch)
END_INTERFACE_MAP()

// {BB765F94-BE9E-11D1-BBC8-204C4F4F5020}
IMPLEMENT_OLECREATE(cicslog, "loginx.cicslog", 0xbb765f94, 0xbe9e, 0x11d1, 0xbb, 0xc8, 0x20, 0x4c, 0x4f, 0x4f, 0x50, 0x20)

/////////////////////////////////////////////////////////////////////////////
// cicslog message handlers

//***************************************************************
// Subroutine to validate a CICS userid and password, and
// optionally have the host assign a four character client
// name.
//***************************************************************

short cicslog::performVerifyPW(LPCTSTR userid, LPCTSTR password, LPCTSTR system)
 
{
	// TODO: Add your dispatch handler code here
	// variable to hold CICS abend codes
	CString abend_code;
	CString t_user;
	CString t_pw;

	// initialize the return code to normal return
	short rc = 0;

	// CICS return code from Verify Password
	CString cicsRC = _T("");

	// First, allocate a CclBuf object and get a pointer to the COMMAREA 
	CclBuf buffer(sizeof(VERIFYCOMM), CclBuf::fixed);

	// Pointer to CICS Commarea for the CICS VERIFYPW program
	VERIFYCOMM* pComm = (VERIFYCOMM*) buffer.dataArea();

	// Build the CICS COMMAREA

	// Get the input variables into CStrings
	t_user = userid;
	t_pw = password;

	// Insert the Userid and Password into the COMMAREA and set the version to "00"
	memcpy(pComm->VERSION, "00", sizeof(pComm->VERSION));
	CLoginxApp::move_comm(t_user, pComm->CICSUSER, sizeof(pComm->CICSUSER), ALPHA_FIELD);
	CLoginxApp::move_comm(t_pw, pComm->CICSPW, sizeof(pComm->CICSPW), ALPHA_FIELD);

	// Get the client name from the instance variable
	CLoginxApp::move_comm(m_clientname, pComm->CLIENTNAME, sizeof(pComm->CLIENTNAME), ALPHA_FIELD);
	
	// Build a connection, buffer area and a flow object
	CclConn connection(system, userid, password);
	CclFlow sflow(Ccl::sync);

	// Run the CICS program to verify the userid and password

	try {	// Catch block for CclClass exceptions
			connection.link(sflow, "VERIFYPW", &buffer);

			// Get the CICS return code from Verify Password
			CLoginxApp::comm_move(cicsRC, pComm->RESP, sizeof(pComm->RESP));

			// Get the Client Name
			CLoginxApp::comm_move(m_clientname, pComm->CLIENTNAME, sizeof(pComm->CLIENTNAME));

			// Clear the current CICS error message
			m_errmsg.Empty();

			// Check if the userid and password were verified
			if (memcmp(cicsRC, "00000", 5) != 0)
			{
				// First, indicate an error in case the atoi doesn't work 
				rc = 100;

				// Pass back the EXEC CICS VERIFY PASSWORD RESP value
				rc = atoi(cicsRC);

				// Set the error message
				m_errmsg = "Userid or Password is invalid";
			}
	    }    
		catch ( CclException& exception ) // Catch CclClass exceptions
		{
			// Get the CICS error message
			m_errmsg = exception.exCodeText();
			rc = exception.exCode();
			abend_code = cicsECI.m_Cclabcode;
        } // end catch

	return rc;
}

//***************************************************************
// Subroutine to get a CICS Server name.  The server names are
// defined in the CICSCLI.INI file.
//***************************************************************

BSTR cicslog::getServerName(short idx) 

{
	CString strResult;
	// TODO: Add your dispatch handler code here

	// Initialize the result
	strResult.Empty();
	m_errmsg = "";

	try {
		// Get the server name
		strResult = cicsECI.serverName(idx);
	} 
	catch (CclException& exception) { // Catch ECI errors

		// Ignore any exceptions which are raised
		m_errmsg = exception.exCodeText();
	}

	return strResult.AllocSysString();
}

//***************************************************************
// Subroutine to get the description for a CICS Server.  The
// description is from the CICSCLI.INI file, in the SERVER
// entry.
//***************************************************************

BSTR cicslog::getServerDesc(short idx) 

{
	CString strResult;
	// TODO: Add your dispatch handler code here

	// Initialize the result
	strResult.Empty();
	m_errmsg = "";

	try {
		// Get the server name
		strResult = cicsECI.serverDesc(idx);
	} 
	catch (CclException& exception) { // Catch ECI errors

		// Ignore any exceptions which are raised
		m_errmsg = exception.exCodeText();
	}

	return strResult.AllocSysString();
}

//***************************************************************
// Subroutine to skip blanks in a string
//***************************************************************

char*  skipblanks(char *temptr)

{
	// skip leading blanks
	while (temptr[0] == ' ')
	{
		temptr++;
	}

	return temptr;
}

//***************************************************************
// Subroutine to look for a CICS client name in the CICSCLI.INI
//  client file, usually found in the CICSCLI\BIN directory.
//***************************************************************

BSTR cicslog::GetClientName() 

{
	// TODO: Add your dispatch handler code here
	FILE *inifile;
	char iniFileLine[MAX_LINE_LEN];
	char clname[5];
	char* temptr;
	CString clientName;

	// Clear the client name
	clientName.Empty();
	memset(clname, '\0', sizeof(clname));

	// see if we can find and open the CICSCLI.INI file
	if ((inifile = fopen(INIFILENAME, "r")) != NULL)
	{
		while (1)
		{
			// read the next line in the source file 
			if (fgets(iniFileLine, MAX_LINE_LEN, inifile) == NULL)
			{
				 break;
			}

			// Check for comments
			if (iniFileLine[0] != ';')
			{
				temptr = iniFileLine;
				temptr = skipblanks(temptr);

				if (memcmp(temptr, "Client ", 7) == 0)
				{
					temptr += 7;
					temptr = skipblanks(temptr);

					if (temptr[0] == '=')
					{
						temptr++;
						temptr = skipblanks(temptr);

						if (temptr[0] != '*')
						{
							memcpy(clname, temptr, 4);
							clientName = clname;
						}

						break;
					}
				}

			}
		}

		fclose(inifile);
	}

	return clientName.AllocSysString();
}

//***********************************************
//  End of file
//***********************************************
